import dis, sys, os, subprocess, re

from pygdb import mi

if sys.argv[1] == '-c':
    g = mi.Gdb()
    proc = subprocess.Popen(["file", sys.argv[2]], stdout=subprocess.PIPE)
    output = proc.communicate()[0]
    exe = re.search("from '(.*)'", output).group(1)
    g.file(exe)
    g.core(sys.argv[2])
else:
    g = mi.Gdb()
    pid = int(sys.argv[1])
    g.file(os.readlink('/proc/%s/exe' % pid))
    g.attach(pid)
thread_ids = g.thread_list_ids().wait().thread_ids.thread_id

def local_named(locals, name):
    for d in locals:
        if d.name == name:
            return d.value

def evaluate_int(expr):
    value = g.data_evaluate_expression(expr)
    return int(value.wait().value)

class FakeCode(object):
    def __init__(self, co_firstlineno, co_lnotab):
        self.co_firstlineno = co_firstlineno
        self.co_lnotab = co_lnotab

offset = evaluate_int('(int)&(((PyStringObject*)(0))->ob_size)')
int_size = 8 # 64 bit dependency!

def get_string(address_expr):
    size = g.data_read_memory(
        address_expr, 'd', int_size, 1, 1, byte_offset=offset).wait()
    size = int(size.memory[0].data[0])
    if not size: return ''
    s = g.data_read_memory(
        '(char*)((PyStringObject*)%s)->ob_sval' % address_expr, 's', 1, 1, size).wait()
    chars = map(int, s.memory[0].data)
    return ''.join(map(chr, chars))

def lineno():
    firstline = evaluate_int('co->co_firstlineno')
    lnotab = get_string('co->co_lnotab')
    lasti = evaluate_int('f->f_lasti')
    c = FakeCode(firstline, lnotab)
    for addr, lineno in dis.findlinestarts(c):
        if lasti >= addr:
            return lineno

for thread_id in thread_ids:
    print "Thread", thread_id
    g.thread_select(thread_id)
    frames = g.stack_list_frames().wait().stack
    eval_found = False
    count = 0
    for frame in frames:
        if frame.frame.func == 'PyEval_EvalFrameEx':
            eval_found = True
            g.stack_select_frame(frame.frame.level)
            print '%s (%s): %s' % (
                get_string('co->co_filename'), lineno(), get_string('co->co_name'))
        if not eval_found:
            arg_values = g.stack_list_arguments(
                1, frame.frame.level, frame.frame.level).wait().stack_args[0].frame.args
            args = []
            for arg in arg_values:
                args.append(arg.name + '=' + arg.value)
            args = ', '.join(args)
            print '#%s %s in %s (%s) from %s' % (
                frame.frame.level, frame.frame.addr, frame.frame.func, args, frame.frame.file)
            count += 1
    print
